/**
 * @packageDocumentation
 * React components, hooks and utils used to connect to {@link @af-utils/virtual-core#VirtualScroller}
 */

import type { ComponentProps } from 'react';
import { ElementType } from 'react';
import type { ReactNode } from 'react';
import { VirtualScroller } from '@af-utils/virtual-core';
import { VirtualScrollerEvent } from '@af-utils/virtual-core';
import { VirtualScrollerInitialParams } from '@af-utils/virtual-core';
import type { VirtualScrollerScrollElement } from '@af-utils/virtual-core';

/**
 * @public
 * React component.
 * Small abstraction, which in 90% cases allows to avoid hook boilerplate.
 */
export declare const List: <C extends ElementType = "div">(props: ListProps<C> & Omit<ComponentProps<C>, "children" | "ref">) => JSX.Element;

/**
 * @public
 * Props passed to List item
 */
export declare interface ListItemProps {
    model: VirtualScroller;
    /** item index */
    i: number;
    /** links to {@link ListProps.itemData} */
    data?: any;
    offset?: number;
}

/**
 * @public
 * {@link List} component props
 */
export declare interface ListProps<C extends ElementType = "div"> {
    model: VirtualScroller;
    children: ElementType<ListItemProps>;
    /** could be accessed in {@link ListItemProps.data} */
    itemData?: any;
    getKey?: (index: number, itemData: any) => string | number;
    component?: C;
    header?: JSX.Element | null;
    footer?: JSX.Element | null;
}

/**
 * @public
 * Map current visible items
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback
 *
 * @returns Array of currently visible items
 */
export declare const mapVisibleRange: <T>(model: VirtualScroller, cb: (index: number) => T) => T[];

/**
 * @public
 * Map current visible items and provide offset for each item.
 *
 * @remarks
 * Used in scenarios, when each item is absolutely positioned
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param cb - callback
 *
 * @returns Array of currently visible items
 */
export declare const mapVisibleRangeWithOffset: <T>(model: VirtualScroller, cb: (index: number, offset: number) => T) => T[];

/**
 * @public
 * React component.
 * Rerenders only on certain {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)}.
 * Allows to optimize performance.
 */
export declare const Subscription: (props: SubscriptionProps) => ReactNode;

/**
 * @public
 * {@link Subscription} component props
 */
export declare interface SubscriptionProps {
    model: VirtualScroller;
    children: () => ReactNode;
    events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[];
}

/**
 * @public
 * React hook.
 * Rerenders component when one of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} gets emitted.
 *
 * @remarks
 * Usually {@link Subscription} is a better alternative.
 * This hook might be used in {@link https://af-utils.com/virtual/examples/react/hook/grid | grid} scenario.
 */
export declare const useComponentSubscription: (model: VirtualScroller, events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[]) => void;

/**
 * @public
 * React hook.
 * Synchronizes scroller with model.
 *
 * @remarks
 * Should be used in window-scroll cases, otherwise `ref={el => model.setScroller( el )}` is preferrable.
 */
export declare const useScroller: (model: VirtualScroller, scroller: VirtualScrollerScrollElement | null) => void;

/**
 * @public
 * React hook.
 * Allows to subscribe to {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} without unnecessary rerenders.
 *
 * @remarks
 * For example can be used in load-on-demand.
 *
 * @param model - {@link @af-utils/virtual-core#VirtualScroller} instance
 * @param events - array of {@link @af-utils/virtual-core#(VirtualScrollerEvent:variable)} to subscribe
 * @param callBack - callback to be called
 */
export declare const useSubscription: (model: VirtualScroller, events: readonly VirtualScrollerEvent[] | VirtualScrollerEvent[], callBack: () => void) => void;

/**
 * @public
 * React hook.
 * Optimal CSS markup for virtual scroll is not intuitive.
 * Use this hook to avoid unneeded boilerplate.
 *
 * @returns Array of two callback refs (for outer and inner elements).
 *
 * @remarks
 * An example could be found {@link https://af-utils.com/virtual/examples/react/hook/simple | here}
 */
export declare const useSyncedStyles: (model: VirtualScroller) => [
(outerRef: HTMLElement | null) => void,
(innerRef: HTMLElement | null) => void
];

/**
 * @public
 * React hook.
 * Calls {@link useVirtualModel} and synchronizes it with props
 *
 * @remarks
 * {@link @af-utils/virtual-core#VirtualScroller.set} is called internally to syncchronize model with props.
 *
 * @example
 * ```tsx
 * useVirtual({
 *     itemCount: 1000,
 *     estimatedItemSize: 100,
 *     overscanCount: 1
 * });
 * ```
 */
export declare const useVirtual: (params: VirtualScrollerInitialParams) => VirtualScroller;

/**
 * @public
 * React hook.
 * Creates and stores one {@link @af-utils/virtual-core#VirtualScroller} instance.
 * It does not get recreated during component lifecycle.
 *
 * @remarks
 * Normally {@link useVirtual} should be used.
 */
export declare const useVirtualModel: (params: VirtualScrollerInitialParams) => VirtualScroller;

export { }
